{-|
Module      : Gargantext.Utils.SpacyNLP
Description : John Snow NLP API connexion
Copyright   : (c) CNRS, 2017
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Spacy ecosystem: https://github.com/explosion/spaCy

Server to be used: https://gitlab.iscpif.fr/gargantext/spacy-server

-}

module Gargantext.Utils.SpacyNLP (
    spacyRequest
  , spacyTagsToToken
  , spacyDataToTokenTags
  , nlpTokenTags
  ) where

import Data.Aeson (encode)
import Data.Set qualified as Set
import Data.Text ( splitOn, toLower )
import Gargantext.Core.Text.Terms.Multi.PosTagging.Types ( Token(Token) )
import Gargantext.Core.Types (TokenTag(..))
import Gargantext.Prelude
import Gargantext.Utils.SpacyNLP.Types
import Network.HTTP.Simple (parseRequest, httpJSON, setRequestBodyLBS, getResponseBody, Response)
import Network.URI (URI(..))


spacyRequest :: URI -> Text -> IO SpacyData
spacyRequest uri txt = do
  req <- parseRequest $ "POST " <> show (uri { uriPath = "/pos" })
  let request = setRequestBodyLBS (encode $ SpacyRequest txt) req
  result <- httpJSON request :: IO (Response SpacyData)
  pure $ getResponseBody result

----------------------------------------------------------------
spacyTagsToToken :: SpacyTags -> Token
spacyTagsToToken st = Token (st ^. spacyTags_index)
                   (st ^. spacyTags_normalized)
                   (st ^. spacyTags_text)
                   (st ^. spacyTags_lemma)
                   (st ^. spacyTags_head_index)
                   (st ^. spacyTags_char_offset)
                   (Just $ st ^. spacyTags_pos)
                   (Just $ st ^. spacyTags_ent_type)
                   (Just $ st ^. spacyTags_prefix)
                   (Just $ st ^. spacyTags_suffix)

spacyDataToTokenTags :: SpacyData -> [[TokenTag]]
spacyDataToTokenTags (SpacyData ds) =
  map (\(SpacyText _ tags) -> spacyTagsToTokenTag <$> tags) ds


spacyTagsToTokenTag :: SpacyTags -> TokenTag
spacyTagsToTokenTag st = TokenTag { _my_token_word = split' (st ^. spacyTags_normalized)
                                  , _my_token_lemma = Set.fromList (split' $ st ^. spacyTags_lemma)
                                  , _my_token_pos = Just $ st ^. spacyTags_pos
                                  , _my_token_ner = Just $ st ^. spacyTags_ent_type
                                  , _my_token_offset_begin = 0 -- TODO
                                  , _my_token_offset_end = 0 -- TODO
                                  }
  where
    split' :: Text -> [Text]
    split' = splitOn " " . Data.Text.toLower
-----------------------------------------------------------------

nlpTokenTags :: URI -> Text -> IO [[TokenTag]]
nlpTokenTags uri txt = spacyDataToTokenTags <$> spacyRequest uri txt
